#!/usr/bin/python

"""
Electric Guitar Instrument for Csound Python Extention
SEE csound.__doc__
"""


import csound


class theremin(csound.Instrument):
	"""
	A theremin is an instrument made of electronic wave synthesizer which has a
	pitch and volume changable proportional to hand-antenna distance/position.
	It cannot stop except by setting the volume to zero (must be implemented in
	the song). The frequency and volume are piecewise continuous which means that
	they cannot skip.
	"""
	def __init__(self, sName='theremin', lExtents=None):
		# tDuration, fAmplitude1, fAmplitude2, fFrequency1, fFrequency2, fWobble, fNoise
		if lExtents == None:
			lExtents = [csound.Extent(0.01, 100, None), # tDuration
				csound.Extent(100, 40000, None),        # fAmplitude1
				csound.Extent(100, 40000, None),        # fAmplitude2
				csound.Extent(10, 2000, None),          # fFrequency1
				csound.Extent(10, 2000, None),          # fFrequency2
				csound.Extent(0, 0.6, None),            # fWobble
				csound.Extent(0, 0.6, None)]            # fNoise
		#end if
		self.prevFrequency = None
		self.prevAmplitude = None
		csound.Instrument.__init__(self, sName, lExtents)
	#end def __init__(iInstrument [, sName])

	def playNote(self, tStart, tDuration, fAmplitude, fFrequency, fWobble, fNoise):
		""" Creates a line to send to CSound. """
		if fDuration == 0: return None # no rest for theremin
		# The Theremin has an initial frequency and amplitude which is set.
		if self.prevFrequency != None:
			fFrequency1 = self.prevFrequency
		else:
			fFrequency1 = fFrequency
		#end if
		fFrequency2 = fFrequency
		if self.prevAmplitude != None:
			fAmplitude1 = self.prevAmplitude
		else:
			fAmplitude1 = fAmplitude
		#end if
		fAmplitude2 = fAmplitude
		# It makes sense to do error checking here.
		if len(self.lExtents) < 7:
			print 'Error: Note extents not correctly formed.'
			return None
		#end if
		if self.lExtents[0].within(tDuration) == False:
			print 'Error: Note duration not within extents.'
			return None
		#end if
		if self.lExtents[1].within(fAmplitude1) == False:
			print 'Error: Note amplitude not within extents.'
			return None
		#end if
		if self.lExtents[2].within(fAmplitude2) == False:
			print 'Error: Note amplitude not within extents.'
			return None
		#end if
		if self.lExtents[3].within(fFrequency1) == False:
			print 'Error: Note frequency not within extents.', self.lExtents[2].fMin, ' !< ', fFrequency1, ' !< ', self.lExtents[2].fMax
			return None
		#end if
		if self.lExtents[4].within(fFrequency2) == False:
			print 'Error: Note frequency not within extents.', self.lExtents[3].fMin, ' !< ', fFrequency2, ' !< ', self.lExtents[3].fMax
			return None
		#end if
		if self.lExtents[5].within(fWobble) == False:
			print 'Error: Note wobble not within extents.'
			return None
		#end if
		if self.lExtents[6].within(fNoise) == False:
			print 'Error: Note noise not within extents.'
			return None
		#end if
		# The Theremin has an initial frequency and amplitude which is set.
		self.prevFrequency = fFrequency2
		self.prevAmplitude = fAmplitude2
		return [self.iInstrument, tStart, tDuration, fAmplitude1, fAmplitude2, fFrequency1, fFrequency2, fWobble, fNoise]
	#end def playNote(tStart, tDuration, fAmplitude, fFrequency, fWobble, fNoise)

	def playRandomNote(self, tStart):
		"""
		Plays a random note based on extents.
		"""
		lReturn = [self.iInstrument, tStart]
		for oExtent in self.lExtents:
			lReturn.append(oExtent.randomWithin()) # very smooth
		#next oExtent
		if lReturn[2] == 0: return None # no rest for theremin
		# The Theremin has an initial frequency which is set.
		if (self.prevFrequency != None):
			lReturn[5] = self.prevFrequency # fFrequency1
		#end if
		# The Theremin has an initial frequency which is set.
		self.prevFrequency = lReturn[6] #fFrequency2
		# The Theremin has an initial amplitude which is set.
		if (self.prevAmplitude != None):
			lReturn[3] = self.prevAmplitude # fAmplitude1
		#end if
		# The Theremin has an initial amplitude which is set.
		self.prevAmplitude = lReturn[4] #fAmplitude2
		return lReturn
	#end def playNote(tStart, tDuration)

	def printOutput(self):
		"""
		Used by Orchestra to make an Orchestra File (*.orc) from instruments.
		Really important is the list of instructions.
		"""
		print 'instr', self.iInstrument, ';', self.sName
		print """
    ; p4 = amplitude1
    ; p5 = amplitude2
    ; p6 = pitch1
    ; p7 = pitch2
    ; p8 = wobble
    ; p9 = noise

	; theremin from amplitude1 to amplitude2
	k4 line p4, p3, p5

	; theremin from pitch1 to pitch2
	k3 line p6, p3, p7

    ; wobble
    k1  oscil p8 * k4, k3 / 5, 1

    ; noise
    k2	rand  (k4 + k1) * p9

    a1	oscil k4 + k1 + k2, k3, 1
        out   a1
	"""
		print 'endin;', self.sName
		print
	#end def printOutput()
#end class theremin(csound.Instrument)

def main():
	"""
	A test function for csound extention.
	"""
	print '<!--', csound.__doc__, '-->'
	oTest = theremin()
	oTest.printOutput()
	oPatch = csound.Patch()
	oPatch.addNote(oTest.playNote(1,    0.25, 10000, 440, 0.2, 0.3))
	oPatch.addNote(oTest.playNote(1.25,    2,  5000, 540, 0.2, 0.3))
	oPatch.printOutput()
#end def main()

if __name__ == '__main__':
	main()
#end if


